#include "stdafx.h"
#include "serial.h"

//------------------------------------------------------------------------------
const char
	*serial_port_names[9]=
   {"COM1","COM2","COM3","COM4","COM5","COM6","COM7","COM8","COM9"};
//------------------------------------------------------------------------------
HANDLE
  hCom=NULL;
CWnd
  *OurParent=NULL;
//------------------------------------------------------------------------------
int Serial_Init(int port, int baud_rate, CWnd* pParent)
	{
	DCB
		dcb;
	BOOL
		fSuccess;

  OurParent=pParent;
	if(hCom)
		CloseHandle(hCom);


	hCom = CreateFile(
		serial_port_names[port],
		GENERIC_READ | GENERIC_WRITE,
		0,             /* comm devices must be opened w/exclusive-access */
		NULL,          /* no security attrs */
		OPEN_EXISTING, /* comm devices must use OPEN_EXISTING */
		0,             /* not overlapped I/O */
		NULL           /* hTemplate must be NULL for comm devices */
		);

	if(hCom == INVALID_HANDLE_VALUE)
		return(1);

	/*
	 * Omit the call to SetupComm to use the default queue sizes.
	 * Get the current configuration.
	 */
	fSuccess = GetCommState(hCom, &dcb);
	if(!fSuccess)
		return(2);

  int
    baud_rates[5]=
      {1200,2400,4800,9600,19200};

	/* Fill in the DCB: baud=9600, 8 data bits, no parity, 1 stop bit. */
	dcb.BaudRate = baud_rates[baud_rate];
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.fRtsControl = RTS_CONTROL_ENABLE;
	dcb.fDtrControl = DTR_CONTROL_ENABLE; 
	fSuccess = SetCommState(hCom, &dcb);

	if(!fSuccess)
		return(3);

	COMMTIMEOUTS
		our_timeouts;

	if(TRUE!=GetCommTimeouts(
			hCom,						// handle of communications device
			&our_timeouts))		// address of comm. time-outs structure
		return(4);

	our_timeouts.ReadIntervalTimeout=0;
	our_timeouts.ReadTotalTimeoutMultiplier=1000;  // 1 second
	our_timeouts.ReadTotalTimeoutConstant=0;

	if(TRUE!=SetCommTimeouts(
			hCom,			    	// handle of communications device
			&our_timeouts)) 	// address of comm. time-outs structure
		return(5);

	return(0);
	}
//------------------------------------------------------------------------------
void Uninit_Serial()
  {
	CloseHandle(hCom);
	hCom=NULL;
  }
//------------------------------------------------------------------------------

unsigned char
	buffer[10000];
DWORD
  buffer_count;
void Clear_Buffer(void)
	{
	buffer_count=0;
  }

void Buffer_Character(unsigned char datum)
	{
  buffer[buffer_count++]=datum;
  if(256<=buffer_count)
    Send_Buffer();
  }

void Buffer_String(char *input)
	{
  char
    this_character; 
	while((this_character=*input++))
    Buffer_Character(this_character);
	}

void Buffer_Characters(unsigned char count,
		  					       unsigned char k0,
											 unsigned char k1,
											 unsigned char k2,
											 unsigned char k3,
											 unsigned char k4,
											 unsigned char k5,
											 unsigned char k6,
											 unsigned char k7,
											 unsigned char k8,
											 unsigned char k9)
	{
	if(0<count)
    Buffer_Character(k0);
	if(1<count)
    Buffer_Character(k1);
	if(2<count)
    Buffer_Character(k2);
	if(3<count)
    Buffer_Character(k3);
	if(4<count)
    Buffer_Character(k4);
	if(5<count)
    Buffer_Character(k5);
	if(6<count)
    Buffer_Character(k6);
	if(7<count)
    Buffer_Character(k7);
	if(8<count)
    Buffer_Character(k8);
	if(9<count)
    Buffer_Character(k9);
	}

  
void Send_Buffer(void)
	{
  if(buffer_count)
		{
		DWORD
			bytes_written;
		bytes_written=0;
		char
	    status_string[200];
	
		if(TRUE!=WriteFile(
				hCom,						// handle of file to write to
				&buffer,     			// address of data to write to file
				buffer_count,    	// number of bytes to write
				&bytes_written,	  // address of number of bytes written
				NULL))	 		 			// addr. of structure needed for overlapped I/O
			{
			strcpy(status_string,"Send_Buffer(): WriteFile() failed.");
			OurParent->MessageBox(status_string,"Error", MB_ICONEXCLAMATION);
			}
		else
			if(bytes_written!=buffer_count)
				{
				sprintf(status_string,"Send_Buffer(): only %d of %d bytes sent.",
								bytes_written,1);
				OurParent->MessageBox(status_string,"Error", MB_ICONEXCLAMATION);
  			}
		buffer_count=0;
		}
  }
//------------------------------------------------------------------------------
void Send_Character(unsigned char datum)
	{
	DWORD
		bytes_written;
	bytes_written=0;
	char
    status_string[200];

	if(TRUE!=WriteFile(
			hCom,							// handle of file to write to
			&datum,      			// address of data to write to file
			1,              	// number of bytes to write
			&bytes_written,	  // address of number of bytes written
			NULL))	 		 			// addr. of structure needed for overlapped I/O
		{
		strcpy(status_string,"WRITE: WriteFile() failed.");
		OurParent->MessageBox(status_string,"Error", MB_ICONEXCLAMATION);
		return;
		}
	if(bytes_written!=1)
		{
		sprintf(status_string,"WRITE: only %d of %d bytes sent.",
				  bytes_written,1);
		OurParent->MessageBox(status_string,"Error", MB_ICONEXCLAMATION);
		return;
		}
	}
//------------------------------------------------------------------------------
void set_dtr_rts(int dtr, int rts)
	{
	DCB
		dcb;
	BOOL
		fSuccess;

	if(!hCom)
		{
		OurParent->MessageBox("No port is open, so I can't set DTR or RTS.",
                "Error", MB_ICONINFORMATION);
		return;
		}

	fSuccess = GetCommState(hCom, &dcb);
	if(!fSuccess)
		{
		OurParent->MessageBox("Can't get the port state.",
                "Error", MB_ICONEXCLAMATION);
		return;
		}

	dcb.fRtsControl = rts?RTS_CONTROL_ENABLE:RTS_CONTROL_DISABLE;
	dcb.fDtrControl = dtr?DTR_CONTROL_ENABLE:DTR_CONTROL_DISABLE; 
	fSuccess = SetCommState(hCom, &dcb);

	if(!fSuccess)
		OurParent->MessageBox("Can't set the port state.",
                "Error", MB_ICONEXCLAMATION);
	}
//------------------------------------------------------------------------------
